import requests
import json

from .habanero_utils import switch_classes,make_ua
from .cn_formats import *

def CNRequest(url, ids = None, format = None, style = None,
        locale = None, **kwargs):

  if(ids.__class__.__name__ == "str"):
    ids = ids.split()
  if(ids.__class__.__name__ == "int"):
    ids = [ids]

  if(len(ids) == 1):
    return make_request(url, ids[0], format, style, locale, **kwargs)
  else:
    coll = []
    for i in range(len(ids)):
      tt = make_request(url, ids[i], format, style, locale, **kwargs)
      coll.append(tt)

    if len(coll) == 1:
      coll = coll[0]
    return coll

def make_request(url, ids, format, style, locale, **kwargs):
  type = cn_format_headers[format]
  htype = {'Accept': type}
  head = dict(make_ua(), **htype)

  if format == "citeproc-json":
    url = "http://api.crossref.org/works/" + ids + "/" + type
    return requests.get(url, headers = head, allow_redirects = True, **kwargs).text
  else:
    if format == "text":
      type = type + "; style = " + style + "; locale = " + locale
    url = url + "/" + ids
    return requests.get(url, headers = head, allow_redirects = True, **kwargs).text




import requests
import json
import re

from .filterhandler import filter_handler
from .habanero_utils import switch_classes,check_json,is_json,parse_json_err,make_ua,filter_dict,rename_query_filters
from .exceptions import *
from .request_class import Request

def request(url, path, ids = None, query = None, filter = None,
        offset = None, limit = None, sample = None, sort = None,
        order = None, facet = None, works = None,
        cursor = None, cursor_max = None, agency = False, **kwargs):

  url = url + path

  if cursor_max.__class__.__name__ != 'NoneType':
    if cursor_max.__class__ != int:
      raise ValueError("cursor_max must be of class int")

  filt = filter_handler(filter)

  payload = {'query':query, 'filter':filt, 'offset':offset,
             'rows':limit, 'sample':sample, 'sort':sort,
             'order':order, 'facet':facet, 'cursor':cursor}
  payload = dict((k, v) for k, v in payload.items() if v)
  # add query filters
  payload.update(filter_dict(kwargs))
  # rename query filters
  payload = rename_query_filters(payload)

  if(ids.__class__.__name__ == 'NoneType'):
    url = url.strip("/")
    try:
      r = requests.get(url, params = payload, headers = make_ua())
      r.raise_for_status()
    except requests.exceptions.HTTPError:
      if is_json(r):
        raise RequestError(r.status_code, parse_json_err(r))
      else:
        r.raise_for_status()
    except requests.exceptions.RequestException as e:
      print(e)
    check_json(r)
    coll = r.json()
    # coll = switch_classes(js, path, works)
  else:
    if(ids.__class__.__name__ == "str"):
      ids = ids.split()
    if(ids.__class__.__name__ == "int"):
      ids = [ids]
    coll = []
    for i in range(len(ids)):
      if works:
        res = Request(url, str(ids[i]) + "/works",
          query, filter, offset, limit, sample, sort,
          order, facet, cursor, cursor_max, **kwargs).do_request()
        coll.append(res)
      else:
        if agency:
          endpt = url + str(ids[i]) + "/agency"
        else:
          endpt = url + str(ids[i])

        endpt = endpt.strip("/")

        try:
          r = requests.get(endpt, params = payload, headers = make_ua())
          r.raise_for_status()
        except requests.exceptions.HTTPError:
          if is_json(r):
            raise RequestError(r.status_code, parse_json_err(r))
          else:
            r.raise_for_status()
        except requests.exceptions.RequestException as e:
          print(e)
        check_json(r)
        js = r.json()
        #tt_out = switch_classes(js, path, works)
        coll.append(js)

    if len(coll) == 1:
      coll = coll[0]

  return coll



import requests
import json
import re

from .filterhandler import filter_handler
from .habanero_utils import switch_classes,check_json,is_json,parse_json_err,make_ua,filter_dict,rename_query_filters
from .exceptions import *

class Request(object):
  '''
  Habanero: request class

  This is the request class for all requests
  '''
  def __init__(self, url, path, query = None, filter = None,
        offset = None, limit = None, sample = None, sort = None,
        order = None, facet = None, cursor = None, cursor_max = None,
        agency = False, **kwargs):
    self.url = url
    self.path = path
    self.query = query
    self.filter = filter
    self.offset = offset
    self.limit = limit
    self.sample = sample
    self.sort = sort
    self.order = order
    self.facet = facet
    self.cursor = cursor
    self.cursor_max = cursor_max
    self.agency = agency
    self.kwargs = kwargs

  def _url(self):
    tmpurl = self.url + self.path
    return tmpurl.strip("/")

  def do_request(self):
    filt = filter_handler(self.filter)

    if self.cursor_max.__class__.__name__ != 'NoneType':
      if self.cursor_max.__class__ != int:
        raise ValueError("cursor_max must be of class int")

    payload = {'query':self.query, 'filter':filt, 'offset':self.offset,
               'rows':self.limit, 'sample':self.sample, 'sort':self.sort,
               'order':self.order, 'facet':self.facet, 'cursor':self.cursor}
    payload = dict((k, v) for k, v in payload.items() if v)
    # add query filters
    payload.update(filter_dict(self.kwargs))
    # rename query filters
    payload = rename_query_filters(payload)

    js = self._req(payload = payload)
    cu = js['message'].get('next-cursor')
    max_avail = js['message']['total-results']
    res = self._redo_req(js, payload, cu, max_avail)
    return res

  def _redo_req(self, js, payload, cu, max_avail):
    if(cu.__class__.__name__ != 'NoneType' and self.cursor_max > len(js['message']['items'])):
      res = [js]
      total = len(js['message']['items'])
      while(cu.__class__.__name__ != 'NoneType' and self.cursor_max > total and total < max_avail):
        payload['cursor'] = cu
        out = self._req(payload = payload)
        # out = _req(url = url, payload = payload)
        cu = out['message'].get('next-cursor')
        res.append(out)
        total = sum([ len(z['message']['items']) for z in res ])
      return res
    else:
      return js

  def _req(self, payload):
    try:
      r = requests.get(self._url(), params = payload, headers = make_ua())
      r.raise_for_status()
    except requests.exceptions.HTTPError:
      if is_json(r):
        raise RequestError(r.status_code, parse_json_err(r))
      else:
        r.raise_for_status()
    except requests.exceptions.RequestException as e:
      print(e)
    check_json(r)
    return r.json()



import re
import requests
from . import __version__

from .response import Works
from .noworks import NoWorks

# helpers ----------
def converter(x):
  if(x.__class__.__name__ == 'str'):
      return [x]
  else:
      return x

def sub_str(x, n = 3):
  if(x.__class__.__name__ == 'NoneType'):
    pass
  else:
    return str(x[:n]) + '***'

def switch_classes(x, path, works):
  if works or re.sub("/", "", path) == "works" and re.sub("/", "", path) != "licenses":
  	return Works(result = x)
  else:
  	return NoWorks(result = x)

def check_kwargs(keys, kwargs):
  for x in range(len(keys)):
    if keys[x] in kwargs.keys():
      mssg = "The %s parameter is not allowed with this method" % keys[x]
      raise Exception(mssg)

def check_json(x):
  ctype = x.headers['Content-Type']
  matched = re.match("application/json", ctype)
  if matched.__class__.__name__ == 'NoneType':
    scode = x.status_code
    if str(x.text) == "Not implemented.":
      scode = 400
    raise RequestError(scode, str(x.text))

def is_json(x):
  if re.search('json', x.headers['Content-Type']).__class__.__name__ == 'NoneType':
    return False
  else:
    return True

def parse_json_err(x):
  return x.json()['message'][0]['message']

def make_ua():
    requa = 'python-requests/' + requests.__version__
    habua = 'habanero/%s' % __version__
    ua = requa + ' ' + habua
    str = {
      'User-Agent': ua,
      'X-USER-AGENT': ua
    }
    return str

def filter_dict(x):
  return dict((k, x[k]) for k, v in x.items() if k.find('query_') == 0 )

def rename_query_filters(x):
  newkeys = [ re.sub('container_title', 'container-title', v) for v in x ]
  newkeys = [ re.sub('query_', 'query.', v) for v in newkeys ]
  mapping = dict(zip(x.keys(), newkeys))
  return { mapping[k]: v for k, v in x.items() }



import sys
import requests
from ..request import request
from ..request_class import Request
from ..habanero_utils import sub_str,check_kwargs
from .filters import filter_names, filter_details

class Crossref(object):
    '''
    Crossref: Class for Crossref search API methods

    Includes methods matching Crossref API routes:

    * /works - :func:`~habanero.Crossref.works`
    * /members - :func:`~habanero.Crossref.members`
    * /prefixes - :func:`~habanero.Crossref.prefixes`
    * /funders - :func:`~habanero.Crossref.funders`
    * /journals - :func:`~habanero.Crossref.journals`
    * /types - :func:`~habanero.Crossref.types`
    * /licenses - :func:`~habanero.Crossref.licenses`

    Also:

    * registration_agency - :func:`~habanero.Crossref.registration_agency`
    * random_dois - :func:`~habanero.Crossref.random_dois`

    What am I actually searching when using the Crossref search API?:

    You are using the Crossref search API described at
    https://github.com/CrossRef/rest-api-doc/blob/master/rest_api.md.
    When you search with query terms, on Crossref servers they are not
    searching full text, or even abstracts of articles, but only what is
    available in the data that is returned to you. That is, they search
    article titles, authors, etc. For some discussion on this, see
    https://github.com/CrossRef/rest-api-doc/issues/101

    Doing setup::

        from habanero import Crossref
        cr = Crossref()
        # set a different base url
        Crossref(base_url = "http://some.other.url")
        # set an api key
        Crossref(api_key = "123456")

    '''
    def __init__(self, base_url = "http://api.crossref.org", api_key = None):

        self.base_url = base_url
        self.api_key = api_key

    def __repr__(self):
      return """< %s \nURL: %s\nKEY: %s\n>""" % (type(self).__name__,
        self.base_url, sub_str(self.api_key))

    def works(self, ids = None, query = None, filter = None, offset = None,
              limit = None, sample = None, sort = None,
              order = None, facet = None, cursor = None,
              cursor_max = 5000, **kwargs):
        '''
        Search Crossref works

        :param ids: [Array] DOIs (digital object identifier) or other identifiers
        :param query: [String] A query string
        :param filter: [Hash] Filter options. See ...
        :param offset: [Fixnum] Number of record to start at, from 1 to infinity.
        :param limit: [Fixnum] Number of results to return. Not relavant when searching with specific dois. Default: 20. Max: 1000
        :param sample: [Fixnum] Number of random results to return. when you use the sample parameter,
            the limit and offset parameters are ignored.
        :param sort: [String] Field to sort on, one of score, relevance,
            updated (date of most recent change to metadata. Currently the same as deposited),
            deposited (time of most recent deposit), indexed (time of most recent index), or
            published (publication date). Note: If the API call includes a query, then the sort
            order will be by the relevance score. If no query is included, then the sort order
            will be by DOI update date.
        :param order: [String] Sort order, one of 'asc' or 'desc'
        :param facet: [Boolean] Include facet results. Default: false
        :param cursor: [String] Cursor character string to do deep paging. Default is None.
            Pass in '*' to start deep paging. Any combination of query, filters and facets may be
            used with deep paging cursors. While rows may be specified along with cursor, offset
            and sample cannot be used.
            See https://github.com/CrossRef/rest-api-doc/blob/master/rest_api.md#deep-paging-with-cursors
        :param cursor_max: [Fixnum] Max records to retrieve. Only used when cursor param used. Because
            deep paging can result in continuous requests until all are retrieved, use this
            parameter to set a maximum number of records. Of course, if there are less records
            found than this value, you will get only those found.
        :param kwargs: any additional arguments will be passed on to
            `requests.get`

        :return: A dict

        Usage::

            from habanero import Crossref
            cr = Crossref()
            cr.works()
            cr.works(ids = '10.1371/journal.pone.0033693')
            dois = ['10.1371/journal.pone.0033693', ]
            cr.works(ids = dois)
            x = cr.works(query = "ecology")
            x['status']
            x['message-type']
            x['message-version']
            x['message']
            x['message']['total-results']
            x['message']['items-per-page']
            x['message']['query']
            x['message']['items']

            # Get full text links
            x = cr.works(filter = {'has_full_text': True})
            x

            # Parse output to various data pieces
            x = cr.works(filter = {'has_full_text': True})
            ## get doi for each item
            [ z['DOI'] for z in x['message']['items'] ]
            ## get doi and url for each item
            [ {"doi": z['DOI'], "url": z['URL']} for z in x['message']['items'] ]
            ### print every doi
            for i in x['message']['items']:
                 print i['DOI']

            # filters - pass in as a dict
            ## see https://github.com/CrossRef/rest-api-doc/blob/master/rest_api.md#filter-names
            cr.works(filter = {'has_full_text': True})
            cr.works(filter = {'has_funder': True, 'has_full_text': True})
            cr.works(filter = {'award_number': 'CBET-0756451', 'award_funder': '10.13039/100000001'})

            # Deep paging, using the cursor parameter
            ## this search should lead to only ~215 results
            cr.works(query = "widget", cursor = "*", cursor_max = 100)
            ## this search should lead to only ~2500 results, in chunks of 500
            res = cr.works(query = "octopus", cursor = "*", limit = 500)
            sum([ len(z['message']['items']) for z in res ])
            ## about 167 results
            res = cr.works(query = "extravagant", cursor = "*", limit = 50, cursor_max = 500)
            sum([ len(z['message']['items']) for z in res ])
            ## cursor_max to get back only a maximum set of results
            res = cr.works(query = "widget", cursor = "*", cursor_max = 100)
            sum([ len(z['message']['items']) for z in res ])
            ## cursor_max - especially useful when a request could be very large
            ### e.g., "ecology" results in ~275K records, lets max at 10,000
            ###   with 1000 at a time
            res = cr.works(query = "ecology", cursor = "*", cursor_max = 10000, limit = 1000)
            sum([ len(z['message']['items']) for z in res ])
            items = [ z['message']['items'] for z in res ]
            items = [ item for sublist in items for item in sublist ]
            [ z['DOI'] for z in items ][0:50]

            # field queries
            res = cr.works(query = "ecology", query_author = 'carl boettiger')
            [ x['author'][0]['family'] for x in res['message']['items'] ]
        '''
        if ids.__class__.__name__ != 'NoneType':
            return request(self.base_url, "/works/", ids,
                query, filter, offset, limit, sample, sort,
                order, facet, None, None, None, **kwargs)
        else:
            return Request(self.base_url, "/works/",
              query, filter, offset, limit, sample, sort,
              order, facet, cursor, cursor_max, **kwargs).do_request()

    def members(self, ids = None, query = None, filter = None, offset = None,
              limit = None, sample = None, sort = None,
              order = None, facet = None, works = False,
              cursor = None, cursor_max = 5000, **kwargs):
        '''
        Search Crossref members

        :param ids: [Array] DOIs (digital object identifier) or other identifiers
        :param query: [String] A query string
        :param filter: [Hash] Filter options. See ...
        :param offset: [Fixnum] Number of record to start at, from 1 to infinity.
        :param limit: [Fixnum] Number of results to return. Not relavant when searching with specific dois. Default: 20. Max: 1000
        :param sample: [Fixnum] Number of random results to return. when you use the sample parameter,
            the limit and offset parameters are ignored. This parameter only used when works requested.
        :param sort: [String] Field to sort on, one of score, relevance,
            updated (date of most recent change to metadata. Currently the same as deposited),
            deposited (time of most recent deposit), indexed (time of most recent index), or
            published (publication date). Note: If the API call includes a query, then the sort
            order will be by the relevance score. If no query is included, then the sort order
            will be by DOI update date.
        :param order: [String] Sort order, one of 'asc' or 'desc'
        :param facet: [Boolean] Include facet results. Default: false
        :param works: [Boolean] If true, works returned as well. Default: false
        :param kwargs: any additional arguments will be passed on to
            `requests.get`

        :return: A dict

        Usage::

            from habanero import Crossref
            cr = Crossref()
            cr.members(ids = 98)

            # get works
            res = cr.members(ids = 98, works = True, limit = 3)
            len(res['message']['items'])
            [ z['DOI'] for z in res['message']['items'] ]

            # cursor - deep paging
            res = cr.members(ids = 98, works = True, cursor = "*")
            sum([ len(z['message']['items']) for z in res ])
            items = [ z['message']['items'] for z in res ]
            items = [ item for sublist in items for item in sublist ]
            [ z['DOI'] for z in items ][0:50]

            # field queries
            res = cr.members(ids = 98, works = True, query_author = 'carl boettiger', limit = 7)
            [ x['author'][0]['family'] for x in res['message']['items'] ]
        '''
        return request(self.base_url, "/members/", ids,
            query, filter, offset, limit, sample, sort,
            order, facet, works, cursor, cursor_max, **kwargs)

    def prefixes(self, ids = None, filter = None, offset = None,
              limit = None, sample = None, sort = None,
              order = None, facet = None, works = False,
              cursor = None, cursor_max = 5000, **kwargs):
        '''
        Search Crossref prefixes

        :param ids: [Array] DOIs (digital object identifier) or other identifiers
        :param filter: [Hash] Filter options. See ...
        :param offset: [Fixnum] Number of record to start at, from 1 to infinity.
        :param limit: [Fixnum] Number of results to return. Not relavant when searching with specific dois. Default: 20. Max: 1000
        :param sample: [Fixnum] Number of random results to return. when you use the sample parameter,
            the limit and offset parameters are ignored. This parameter only used when works requested.
        :param sort: [String] Field to sort on, one of score, relevance,
            updated (date of most recent change to metadata. Currently the same as deposited),
            deposited (time of most recent deposit), indexed (time of most recent index), or
            published (publication date). Note: If the API call includes a query, then the sort
            order will be by the relevance score. If no query is included, then the sort order
            will be by DOI update date.
        :param order: [String] Sort order, one of 'asc' or 'desc'
        :param facet: [Boolean] Include facet results. Default: false
        :param works: [Boolean] If true, works returned as well. Default: false
        :param kwargs: any additional arguments will be passed on to
            `requests.get`

        :return: A dict

        Usage::

            from habanero import Crossref
            cr = Crossref()
            cr.prefixes(ids = "10.1016")
            cr.prefixes(ids = ['10.1016','10.1371','10.1023','10.4176','10.1093'])

            # get works
            cr.prefixes(ids = "10.1016", works = True)

            # Limit number of results
            cr.prefixes(ids = "10.1016", works = True, limit = 3)

            # Sort and order
            cr.prefixes(ids = "10.1016", works = True, sort = "relevance", order = "asc")

            # cursor - deep paging
            res = cr.prefixes(ids = "10.1016", works = True, cursor = "*", limit = 200)
            sum([ len(z['message']['items']) for z in res ])
            items = [ z['message']['items'] for z in res ]
            items = [ item for sublist in items for item in sublist ]
            [ z['DOI'] for z in items ][0:50]

            # field queries
            res = cr.prefixes(ids = "10.1371", works = True, query_editor = 'cooper', filter = {'type': 'journal-article'})
            eds = [ x.get('editor') for x in res['message']['items'] ]
            [ z for z in eds if z is not None ]
        '''
        check_kwargs(["query"], kwargs)
        return request(self.base_url, "/prefixes/", ids,
          query = None, filter = filter, offset = offset, limit = limit,
          sample = sample, sort = sort, order = order, facet = facet, works = works,
          cursor = cursor, cursor_max = cursor_max, **kwargs)

    def funders(self, ids = None, query = None, filter = None, offset = None,
              limit = None, sample = None, sort = None,
              order = None, facet = None, works = False,
              cursor = None, cursor_max = 5000, **kwargs):
        '''
        Search Crossref funders

        Note that funders without IDs don't show up on the `/funders` route,
        that is, won't show up in searches via this method

        :param ids: [Array] DOIs (digital object identifier) or other identifiers
        :param query: [String] A query string
        :param filter: [Hash] Filter options. See ...
        :param offset: [Fixnum] Number of record to start at, from 1 to infinity.
        :param limit: [Fixnum] Number of results to return. Not relavant when searching with specific dois. Default: 20. Max: 1000
        :param sample: [Fixnum] Number of random results to return. when you use the sample parameter,
            the limit and offset parameters are ignored. This parameter only used when works requested.
        :param sort: [String] Field to sort on, one of score, relevance,
            updated (date of most recent change to metadata. Currently the same as deposited),
            deposited (time of most recent deposit), indexed (time of most recent index), or
            published (publication date). Note: If the API call includes a query, then the sort
            order will be by the relevance score. If no query is included, then the sort order
            will be by DOI update date.
        :param order: [String] Sort order, one of 'asc' or 'desc'
        :param facet: [Boolean] Include facet results. Default: false
        :param works: [Boolean] If true, works returned as well. Default: false
        :param kwargs: any additional arguments will be passed on to
            `requests.get`

        :return: A dict

        Usage::

            from habanero import Crossref
            cr = Crossref()
            cr.funders(ids = '10.13039/100000001')
            cr.funders(query = "NSF")

            # get works
            cr.funders(ids = '10.13039/100000001', works = True)

            # cursor - deep paging
            res = cr.funders(ids = '10.13039/100000001', works = True, cursor = "*", limit = 200)
            sum([ len(z['message']['items']) for z in res ])
            items = [ z['message']['items'] for z in res ]
            items = [ item for sublist in items for item in sublist ]
            [ z['DOI'] for z in items ][0:50]

            # field queries
            res = cr.funders(ids = "10.13039/100000001", works = True, query_container_title = 'engineering', filter = {'type': 'journal-article'})
            eds = [ x.get('editor') for x in res['message']['items'] ]
            [ z for z in eds if z is not None ]
        '''
        return request(self.base_url, "/funders/", ids,
          query, filter, offset, limit, sample, sort,
          order, facet, works, cursor, cursor_max, **kwargs)

    def journals(self, ids = None, query = None, filter = None, offset = None,
              limit = None, sample = None, sort = None,
              order = None, facet = None, works = False,
              cursor = None, cursor_max = 5000, **kwargs):
        '''
        Search Crossref journals

        :param ids: [Array] DOIs (digital object identifier) or other identifiers
        :param query: [String] A query string
        :param filter: [Hash] Filter options. See ...
        :param offset: [Fixnum] Number of record to start at, from 1 to infinity.
        :param limit: [Fixnum] Number of results to return. Not relavant when searching with specific dois. Default: 20. Max: 1000
        :param sample: [Fixnum] Number of random results to return. when you use the sample parameter,
            the limit and offset parameters are ignored. This parameter only used when works requested.
        :param sort: [String] Field to sort on, one of score, relevance,
            updated (date of most recent change to metadata. Currently the same as deposited),
            deposited (time of most recent deposit), indexed (time of most recent index), or
            published (publication date). Note: If the API call includes a query, then the sort
            order will be by the relevance score. If no query is included, then the sort order
            will be by DOI update date.
        :param order: [String] Sort order, one of 'asc' or 'desc'
        :param facet: [Boolean] Include facet results. Default: false
        :param works: [Boolean] If true, works returned as well. Default: false
        :param kwargs: any additional arguments will be passed on to
            `requests.get`

        :return: A dict

        Usage::

            from habanero import Crossref
            cr = Crossref()
            cr.journals(ids = "2167-8359")
            cr.journals()

            # pass many ids
            cr.journals(ids = ['1803-2427', '2326-4225'])

            # search
            cr.journals(query = "ecology")
            cr.journals(query = "peerj")

            # get works
            cr.journals(ids = "2167-8359", works = True)
            cr.journals(ids = "2167-8359", query = 'ecology', works = True, sort = 'score', order = "asc")
            cr.journals(ids = "2167-8359", query = 'ecology', works = True, sort = 'score', order = "desc")
            cr.journals(ids = "2167-8359", works = True, filter = {'from_pub_date': '2014-03-03'})
            cr.journals(ids = '1803-2427', works = True)
            cr.journals(ids = '1803-2427', works = True, sample = 1)
            cr.journals(limit: 2)

            # cursor - deep paging
            res = cr.funders(ids = '10.13039/100000001', works = True, cursor = "*", limit = 200)
            sum([ len(z['message']['items']) for z in res ])
            items = [ z['message']['items'] for z in res ]
            items = [ item for sublist in items for item in sublist ]
            [ z['DOI'] for z in items ][0:50]

            # field queries
            res = cr.journals(ids = "2167-8359", works = True, query_title = 'fish', filter = {'type': 'journal-article'})
            [ x.get('title') for x in res['message']['items'] ]
        '''
        return request(self.base_url, "/journals/", ids,
          query, filter, offset, limit, sample, sort,
          order, facet, works, cursor, cursor_max, **kwargs)

    def types(self, ids = None, query = None, filter = None, offset = None,
              limit = None, sample = None, sort = None,
              order = None, facet = None, works = False,
              cursor = None, cursor_max = 5000, **kwargs):
        '''
        Search Crossref types

        :param ids: [Array] Type identifier, e.g., journal
        :param query: [String] A query string
        :param filter: [Hash] Filter options. See ...
        :param offset: [Fixnum] Number of record to start at, from 1 to infinity.
        :param limit: [Fixnum] Number of results to return. Not relavant when searching with specific dois. Default: 20. Max: 1000
        :param sample: [Fixnum] Number of random results to return. when you use the sample parameter,
            the limit and offset parameters are ignored. This parameter only used when works requested.
        :param sort: [String] Field to sort on, one of score, relevance,
            updated (date of most recent change to metadata. Currently the same as deposited),
            deposited (time of most recent deposit), indexed (time of most recent index), or
            published (publication date). Note: If the API call includes a query, then the sort
            order will be by the relevance score. If no query is included, then the sort order
            will be by DOI update date.
        :param order: [String] Sort order, one of 'asc' or 'desc'
        :param facet: [Boolean] Include facet results. Default: false
        :param works: [Boolean] If true, works returned as well. Default: false
        :param kwargs: any additional arguments will be passed on to
            `requests.get`

        :return: A dict

        Usage::

            from habanero import Crossref
            cr = Crossref()
            cr.types()
            cr.types(ids = "journal")
            cr.types(ids = "journal-article")
            cr.types(ids = "journal", works = True)

            # field queries
            res = cr.types(ids = "journal-article", works = True, query_title = 'gender', rows = 100)
            [ x.get('title') for x in res['message']['items'] ]
        '''
        return request(self.base_url, "/types/", ids,
            query, filter, offset, limit, sample, sort,
            order, facet, works, cursor, cursor_max, **kwargs)

    def licenses(self, query = None, offset = None,
              limit = None, sample = None, sort = None,
              order = None, facet = None, **kwargs):
        '''
        Search Crossref licenses

        :param query: [String] A query string
        :param offset: [Fixnum] Number of record to start at, from 1 to infinity.
        :param limit: [Fixnum] Number of results to return. Not relavant when searching with specific dois. Default: 20. Max: 1000
        :param sort: [String] Field to sort on, one of score, relevance,
            updated (date of most recent change to metadata. Currently the same as deposited),
            deposited (time of most recent deposit), indexed (time of most recent index), or
            published (publication date). Note: If the API call includes a query, then the sort
            order will be by the relevance score. If no query is included, then the sort order
            will be by DOI update date.
        :param order: [String] Sort order, one of 'asc' or 'desc'
        :param facet: [Boolean] Include facet results. Default: false
        :param kwargs: any additional arguments will be passed on to
            `requests.get`

        :return: A dict

        Usage::

            from habanero import Crossref
            cr = Crossref()
            cr.licenses()
            cr.licenses(query = "creative")
        '''
        check_kwargs(["ids", "filter", "works"], kwargs)
        res = request(self.base_url, "/licenses/", None,
            query, None, offset, limit, None, sort,
            order, facet, None, None, None, None, **kwargs)
        return res

    def registration_agency(self, ids, **kwargs):
        '''
        Determine registration agency for DOIs

        :param ids: [Array] DOIs (digital object identifier) or other identifiers
        :param kwargs: any additional arguments will be passed on to
            `requests.get`

        :return: list of DOI minting agencies

        Usage::

            from habanero import Crossref
            cr = Crossref()
            cr.registration_agency('10.1371/journal.pone.0033693')
            cr.registration_agency(ids = ['10.1007/12080.1874-1746','10.1007/10452.1573-5125', '10.1111/(issn)1442-9993'])
        '''
        check_kwargs(["query", "filter", "offset", "limit", "sample", "sort",
            "order", "facet", "works"], kwargs)
        res = request(self.base_url, "/works/", ids,
            None, None, None, None, None, None,
            None, None, None, None, None, True, **kwargs)
        if res.__class__ != list:
            k = []
            k.append(res)
        else:
            k = res
        return [ z['message']['agency']['label'] for z in k ]

    def random_dois(self, sample = 10, **kwargs):
        '''
        Get a random set of DOIs

        :param sample: [Fixnum] Number of random DOIs to return. Default: 10
        :param kwargs: any additional arguments will be passed on to
            `requests.get`

        :return: [Array] of DOIs

        Usage::

            from habanero import Crossref
            cr = Crossref()
            cr.random_dois(1)
            cr.random_dois(10)
            cr.random_dois(50)
            cr.random_dois(100)
        '''
        res = request(self.base_url, "/works/", None,
            None, None, None, None, sample, None,
            None, None, True, **kwargs)
        return [ z['DOI'] for z in res['message']['items'] ]

    @staticmethod
    def filter_names():
        '''
        Filter names - just the names of each filter

        Filters are used in the Crossref search API to modify searches

        :return: dict

        Usage::

            from habanero import Crossref
            cr = Crossref()
            cr.filter_names()
        '''
        return filter_names

    @staticmethod
    def filter_details():
        '''
        Filter details - filter names, possible values, and description

        Filters are used in the Crossref search API to modify searches

        :return: dict

        Usage::

            from habanero import Crossref
            cr = Crossref()
            cr.filter_details()
            # Get descriptions for each filter
            x = cr.filter_details()
            [ z['description'] for z in x.values() ]
        '''
        return filter_details



import requests
import json
import re

from ..habanero_utils import check_json

def csl_styles(**kwargs):
  '''
  Get list of styles from https://github.com/citation-style-language/styles

  :param kwargs: any additional arguments will be passed on to `requests.get`

  :return: list, of CSL styles

  Usage::

      from habanero import cn
      cn.csl_styles()
  '''
  base = "https://api.github.com/repos/citation-style-language/styles"
  tt = requests.get(base + '/commits?per_page=1', **kwargs)
  tt.raise_for_status()
  check_json(tt)
  commres = tt.json()
  sha = commres[0]['sha']
  sty = requests.get(base + "/git/trees/" + sha, **kwargs)
  sty.raise_for_status()
  check_json(sty)
  res = sty.json()
  files = [ z['path'] for z in res['tree'] ]
  matches = [ re.search(".csl", g) for g in files ]
  csls = [ x.string for x in filter(None, matches) ]
  return [ re.sub(".csl", "", x) for x in csls ]



class Works(object):
  '''
  Habanero: response class

  This is the response wrapper for any works data responses
  '''
  def __init__(self, result):
    self.result = result

  def status(self):
    return self.result['status']

  def message_type(self):
    return self.result['message-type']

  def message_version(self):
    return self.result['message-version']

  def message(self):
    return self.result['message']

  def total_results(self):
      return self.result['message']['total-results']

  def items_per_page(self):
      return self.result['message']['items-per-page']

  def query(self):
      return self.result['message']['query']

  def items(self):
      return self.result['message']['items']

  def links(self):
      tmp = [ z.get('link') for z in self.result['message']['items'] ]
      tmp = [x for x in tmp if x is not None]
      return tmp



class Error(Exception):
    """Base class for exceptions in this module."""
    pass

class RequestError(Error):
    """
    Exception raised for request errors.

    This error occurrs when a request sent to the Crossref API
    results in an error. We give back:

    - HTTP status code
    - Error message
    """

    @property
    def status_code(self):
        return self.args[0]

    @property
    def error(self):
        return self.args[1]

    def __str__(self):
      return '(%s) caused by "%s"' % (
        self.status_code, self.error)

# url = "http://api.crossref.org/works?rows=fart"
# url = "http://api.crossref.org/funders?filter=from-pub-date%3A2014-03-03"
# foobar(url)
# def foobar(x):
#     try:
#         r = requests.get(x)
#         r.raise_for_status()
#     except requests.exceptions.HTTPError:
#         raise RequestError(r.status_code, r.json()['message'][0]['message'])
#     except requests.exceptions.RequestException as e:
#         print e



class NoWorks(object):
  '''
  Habanero: agency class
  '''
  def __init__(self, result):
    self.result = result

  def status(self):
    return self.result['status']

  def message_type(self):
    return self.result['message-type']

  def message_version(self):
    return self.result['message-version']

  def message(self):
    return self.result['message']



# def names():
#   '''
#   Crossref filter names

#   :return: list

#   Usage:
#   >>> from habanero import filters
#   >>> filters.names
#   '''
#   return filter_list

# def details():
#   '''
#   Crossref filters details

#   :return: list

#   Usage:
#   >>> from habanero import filters
#   >>> filters.filters
#   >>> filters.filters['has_funder']
#   >>> filters.filters['has_funder']['description']
#   '''
#   return filter_details

filter_names = [
  'has_funder','funder','prefix','member','from_index_date','until_index_date',
  'from_deposit_date','until_deposit_date','from_update_date','until_update_date',
  'from_first_deposit_date','until_first_deposit_date','from_pub_date','until_pub_date',
  'has_license','license_url','license_version','license_delay','has_full_text',
  'full_text_version','full_text_type','public_references','has_references','has_archive',
  'archive','has_orcid','orcid','issn','type','directory','doi','updates','is_update',
  'has_update_policy','container_title','publisher_name','category_name','type_name',
  'from_created_date', 'until_created_date', 'affiliation', 'has_affiliation',
  'assertion_group', 'assertion', 'article_number', 'alternative_id'
]

filter_details = {
  "has_funder": { "possible_values": None, "description": "metadata which includes one or more funder entry" },
  "funder": { "possible_values": "{funder_id}", "description": "metadata which include the {funder_id} in FundRef data" },
  "prefix": { "possible_values": "{owner_prefix}", "description": "metadata belonging to a DOI owner prefix {owner_prefix} (e.g. '10.1016' )" },
  "member": { "possible_values": "{member_id}", "description": "metadata belonging to a CrossRef member" },
  "from_index_date": { "possible_values": '{date}', "description": "metadata indexed since (inclusive) {date}" },
  "until_index_date": { "possible_values": '{date}', "description": "metadata indexed before (inclusive) {date}" },
  "from_deposit_date": { "possible_values": '{date}', "description": "metadata last (re)deposited since (inclusive) {date}" },
  "until_deposit_date": { "possible_values": '{date}', "description": "metadata last (re)deposited before (inclusive) {date}" },
  "from_update_date": { "possible_values": '{date}', "description": "Metadata updated since (inclusive) {date} Currently the same as 'from_deposit_date'" },
  "until_update_date": { "possible_values": '{date}', "description": "Metadata updated before (inclusive) {date} Currently the same as 'until_deposit_date'" },
  "from_created_date": { "possible_values": '{date}', "description": "metadata first deposited since (inclusive) {date}" },
  "until_created_date": { "possible_values": '{date}', "description": "metadata first deposited before (inclusive) {date}" },
  "from_pub_date": { "possible_values": '{date}', "description": "metadata where published date is since (inclusive) {date}" },
  "until_pub_date": { "possible_values": '{date}', "description": "metadata where published date is before (inclusive)  {date}" },
  "has_license": { "possible_values": None, "description": "metadata that includes any '<license_ref>' elements" },
  "license_url": { "possible_values": '{url}', "description": "metadata where '<license_ref>' value equals {url}" },
  "license_version": { "possible_values": '{string}', "description": "metadata where the '<license_ref>''s 'applies_to' attribute  is '{string}'"},
  "license_delay": { "possible_values": "{integer}", "description": "metadata where difference between publication date and the '<license_ref>''s 'start_date' attribute is <= '{integer}' (in days"},
  "has_full_text": { "possible_values": None, "description": "metadata that includes any full text '<resource>' elements_" },
  "full_text_version": { "possible_values": '{string}' , "description": "metadata where '<resource>' element's 'content_version' attribute is '{string}'" },
  "full_text_type": { "possible_values": '{mime_type}' , "description": "metadata where '<resource>' element's 'content_type' attribute is '{mime_type}' (e.g. 'application/pdf')" },
  "public_references": { "possible_values": None, "description": "metadata where publishers allow references to be distributed publically" },
  "has_references": { "possible_values": None , "description": "metadata for works that have a list of references" },
  "has_archive": { "possible_values": None , "description": "metadata which include name of archive partner" },
  "archive": { "possible_values": '{string}', "description": "metadata which where value of archive partner is '{string}'" },
  "has_orcid": { "possible_values": None, "description": "metadata which includes one or more ORCIDs" },
  "orcid": { "possible_values": '{orcid}', "description": "metadata where '<orcid>' element's value = '{orcid}'" },
  "issn": { "possible_values": '{issn}', "description": "metadata where record has an ISSN = '{issn}' Format is 'xxxx_xxxx'." },
  "type": { "possible_values": '{type}', "description": "metadata records whose type = '{type}' Type must be an ID value from the list of types returned by the '/types' resource" },
  "directory": { "possible_values": "{directory}", "description": "metadata records whose article or serial are mentioned in the given '{directory}'. Currently the only supported value is 'doaj'" },
  "doi": { "possible_values": '{doi}', "description": "metadata describing the DOI '{doi}'" },
  "updates": { "possible_values": '{doi}', "description": "metadata for records that represent editorial updates to the DOI '{doi}'" },
  "is_update": { "possible_values": None, "description": "metadata for records that represent editorial updates" },
  "has_update_policy": { "possible_values": None, "description": "metadata for records that include a link to an editorial update policy" },
  "container_title": { "possible_values": None, "description": "metadata for records with a publication title exactly with an exact match" },
  "publisher_name": { "possible_values": None, "description": "metadata for records with an exact matching publisher name" },
  "category_name": { "possible_values": None, "description": "metadata for records with an exact matching category label" },
  "type_name": { "possible_values": None, "description": "metadata for records with an exacty matching type label" },
  "award_number": { "possible_values": "{award_number}", "description": "metadata for records with a matching award nunber_ Optionally combine with 'award_funder'" },
  "award_funder": { "possible_values": '{funder doi or id}', "description": "metadata for records with an award with matching funder. Optionally combine with 'award_number'" },
  "assertion_group": { "possible_values": None, "description": "metadata for records with an assertion in a particular group" },
  "assertion": { "possible_values": None, "description": "metadata for records with a particular named assertion" },
  "affiliation": { "possible_values": None, "description": "metadata for records with at least one contributor with the given affiliation" },
  "has_affiliation": { "possible_values": None, "description": "metadata for records that have any affiliation information" },
  "alternative_id": { "possible_values": None, "description": "metadata for records with the given alternative ID, which may be a publisher_specific ID, or any other identifier a publisher may have provided" },
  "article_number": { "possible_values": None, "description": "metadata for records with a given article number" }
}



# -*- coding: utf-8 -*-

# habanero

'''
habanero library
~~~~~~~~~~~~~~~~~~~~~

habanero is a low level client for the Crossref search API.

Usage::

   from habanero import Crossref
   cr = Crossref()

   # setup a different base URL
   Crossref(base_url = "http://some.other.url")

   # setup an api key
   Crossref(api_key = "123456")

   # Make request against works route
   cr.works(ids = '10.1371/journal.pone.0033693')

   # curl options
   ## For example, set a timeout
   cr.works(query = "ecology", timeout=0.1)

   ## advanced logging
   ### setup first
   import requests
   import logging
   import httplib as http_client
   http_client.HTTPConnection.debuglevel = 1
   logging.basicConfig()
   logging.getLogger().setLevel(logging.DEBUG)
   requests_log = logging.getLogger("requests.packages.urllib3")
   requests_log.setLevel(logging.DEBUG)
   requests_log.propagate = True
   ### then make request
   cr.works(query = "ecology")
'''

__title__ = 'habanero'
__version__ = '0.2.6'
__author__ = 'Scott Chamberlain'
__license__ = 'MIT'

from .crossref import Crossref
from .cn import content_negotiation, csl_styles
from .counts import citation_count
from .exceptions import *



cn_formats = ["rdf-xml", "turtle", "citeproc-json",
	          "citeproc-json-ish", "text", "ris", "bibtex",
	          "crossref-xml", "datacite-xml", "bibentry",
	          "crossref-tdm"]

cn_format_headers = {"rdf-xml": "application/rdf+xml",
	          "turtle": "text/turtle",
	          "citeproc-json": "transform/application/vnd.citationstyles.csl+json",
	          "text": "text/x-bibliography",
	          "ris": "application/x-research-info-systems",
	          "bibtex": "application/x-bibtex",
	          "crossref-xml": "application/vnd.crossref.unixref+xml",
	          "datacite-xml": "application/vnd.datacite.datacite+xml",
	          "bibentry": "application/x-bibtex",
	          "crossref-tdm": "application/vnd.crossref.unixsd+xml"}

cn_types = {"rdf-xml": "text/xml",
		      "turtle": "text/plain",
		      "citeproc-json": "application/json",
		      "citeproc-json-ish": "application/json",
		      "text": "text/plain",
		      "ris": "text/plain",
		      "bibtex": "text/plain",
		      "crossref-xml": "text/xml",
		      "datacite-xml": "text/xml",
		      "bibentry": "text/plain",
		      "crossref-tdm": "text/xml"}



from ..cnrequest import CNRequest
from .constants import *

def content_negotiation(ids = None, format = "bibtex", style = 'apa',
    locale = "en-US", **kwargs):
    '''
    Get citations in various formats from CrossRef

    :param ids: [str] Search by a single DOI or many DOIs, each a string. If many
        passed in, do so in a list
    :param format: [str] Name of the format. One of "rdf-xml", "turtle", "citeproc-json",
        "citeproc-json-ish", "text", "ris", "bibtex" (Default), "crossref-xml",
        "datacite-xml","bibentry", or "crossref-tdm"
    :param style: [str] A CSL style (for text format only). See :func:`~habanero.cn.csl_styles`
        for options. Default: "apa". If there's a style that CrossRef doesn't support
        you'll get a `(500) Internal Server Error`
    :param locale: [str] Language locale. See `locale.locale_alias`
    :param kwargs: any additional arguments will be passed on to `requests.get`

    :return: string, which can be parsed to various formats depending on what
        format you request (e.g., JSON vs. XML vs. bibtex)

    Usage::

        from habanero import cn
        cn.content_negotiation(ids = '10.1126/science.169.3946.635')

        # get citeproc-json
        cn.content_negotiation(ids = '10.1126/science.169.3946.635', format = "citeproc-json")

        # some other formats
        cn.content_negotiation(ids = "10.1126/science.169.3946.635", format = "rdf-xml")
        cn.content_negotiation(ids = "10.1126/science.169.3946.635", format = "crossref-xml")
        cn.content_negotiation(ids = "10.1126/science.169.3946.635", format = "text")

        # return an R bibentry type
        cn.content_negotiation(ids = "10.1126/science.169.3946.635", format = "bibentry")
        cn.content_negotiation(ids = "10.6084/m9.figshare.97218", format = "bibentry")

        # return an apa style citation
        cn.content_negotiation(ids = "10.1126/science.169.3946.635", format = "text", style = "apa")
        cn.content_negotiation(ids = "10.1126/science.169.3946.635", format = "text", style = "harvard3")
        cn.content_negotiation(ids = "10.1126/science.169.3946.635", format = "text", style = "elsevier-harvard")
        cn.content_negotiation(ids = "10.1126/science.169.3946.635", format = "text", style = "ecoscience")
        cn.content_negotiation(ids = "10.1126/science.169.3946.635", format = "text", style = "heredity")
        cn.content_negotiation(ids = "10.1126/science.169.3946.635", format = "text", style = "oikos")

        # Using DataCite DOIs
        ## some formats don't work
        # cn.content_negotiation(ids = "10.5284/1011335", format = "text")
        # cn.content_negotiation(ids = "10.5284/1011335", format = "crossref-xml")
        # cn.content_negotiation(ids = "10.5284/1011335", format = "crossref-tdm")

        ## But most do work
        cn.content_negotiation(ids = "10.5284/1011335", format = "datacite-xml")
        cn.content_negotiation(ids = "10.5284/1011335", format = "rdf-xml")
        cn.content_negotiation(ids = "10.5284/1011335", format = "turtle")
        cn.content_negotiation(ids = "10.5284/1011335", format = "citeproc-json")
        cn.content_negotiation(ids = "10.5284/1011335", format = "ris")
        cn.content_negotiation(ids = "10.5284/1011335", format = "bibtex")
        cn.content_negotiation(ids = "10.5284/1011335", format = "bibentry")
        cn.content_negotiation(ids = "10.5284/1011335", format = "bibtex")

        # many DOIs
        dois = ['10.5167/UZH-30455','10.5167/UZH-49216','10.5167/UZH-503', '10.5167/UZH-38402','10.5167/UZH-41217']
        x = cn.content_negotiation(ids = dois)
    '''
    return CNRequest(cn_base_url, ids, format, style, locale, **kwargs)



import re

def filter_handler(x = None):
  if(x.__class__.__name__ == 'NoneType'):
    return None
  else:
    # lowercase bools
    for k, v in x.items():
      if v.__class__ == bool:
        x[k] = str(v).lower()

    # combine
    nn = x.keys()
    if any([ i in others for i in nn]):
      out = []
      for i in nn:
        if i in others:
          out.append(switch_filters(i))
        else:
          out.append(i)
      nn = out

    newnn = [ re.sub("_", "-", z) for z in nn ]
    newnnd = dict(zip(x.keys(), newnn))
    x = rename_keys(x, newnnd)
    x = ','.join(['{}:{}'.format(k,v) for k,v in x.items()])
    return x

others = ['license_url','license_version','license_delay',
          'full_text_version','full_text_type','award_number',
          'award_funder']

dict_filts = {
  'license_url': 'license.url',
  'license_version': 'license.version',
  'license_delay': 'license.delay',
  'full_text_version': 'full-text.version',
  'full_text_type': 'full-text.type',
  'award_number': 'award.number',
  'award_funder': 'award.funder'
}

def switch_filters(x):
  return switch(x, dict_filts)

def switch(x, dict):
  return dict[x]

# https://github.com/healthsites/healthsites/blob/3b10b12f004bfa783ee3121647ff9856836717f3/django_project/api/utils.py
def rename_keys(old_dict, transform):
  new_dict = {}
  for k, v in old_dict.items():
      if k in transform:
          new_dict.update({transform[k]: v})
      else:
          new_dict.update({k: v})
  return new_dict



# -*- coding: utf-8 -*-

# counts

'''
citation counts
~~~~~~~~~~~~~~~

Get citation count data

Usage::

   from habanero import counts
   cr = Crossref()
   counts.citation_count(doi = "10.1371/journal.pone.0042793"
'''

from .counts import citation_count



import requests
from xml.dom import minidom
from ..habanero_utils import make_ua

def citation_count(doi, url = "http://www.crossref.org/openurl/",
    key = "cboettig@ropensci.org", **kwargs):
    '''
    Get a citation count with a DOI

    :@param doi: [String] DOI, digital object identifier
    :@param url: [String] the API url for the function (should be left to default)
    :@param keyc [String] your API key

    See http://labs.crossref.org/openurl/ for more info on this Crossref API service.

    Usage::

        from habanero import counts
        counts.citation_count(doi = "10.1371/journal.pone.0042793")
        counts.citation_count(doi = "10.1016/j.fbr.2012.01.001")
        # DOI not found
        ## FIXME
        counts.citation_count(doi = "10.1016/j.fbr.2012")
    '''
    args = {"id": "doi:" + doi, "pid": key, "noredirect": True}
    args = dict((k, v) for k, v in args.items() if v)
    res = requests.get(url, params = args, headers = make_ua(), **kwargs)
    xmldoc = minidom.parseString(res.content)
    val = xmldoc.getElementsByTagName('query')[0].attributes['fl_count'].value
    return int(str(val))
